@@ -4,7 +4,7 @@
 #	Massachusetts Institute of Technology.
 # written by Olivier Thereaux <ot@w3.org> for W3C
 #
-# $Id: Config.pm,v 1.10 2005/09/09 06:33:11 ot Exp $
+# $Id: Config.pm,v 1.12 2007/07/24 06:08:11 ot Exp $
 
 package W3C::LogValidator::Config;
 use strict;
@@ -14,7 +14,7 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw() ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
-our $VERSION = sprintf "%d.%03d",q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/;
+our $VERSION = sprintf "%d.%03d",q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/;
 
 our $config_filename;
 our %conf;
@@ -84,6 +84,10 @@ sub config_default
 		push @{$conf{LogProcessor}{LogFiles}}, "/var/log/apache/access.log";
 		$conf{LogProcessor}{LogType}{"/var/log/apache/access.log"}="common";
 	}
+	if (!exists $conf{LogProcessor}{RefererMatch})
+	{	
+		$conf{LogProcessor}{RefererMatch} =".*";
+	}
 
 	if (!exists $conf{LogProcessor}{UseValidationModule})
 	{
@@ -97,6 +101,11 @@ sub config_default
 	{
 		$conf{LogProcessor}{EntriesPerLogfile}=100000; 
 	}
+	# adding default handling of URIs with query strings
+	if (!exists $conf{LogProcessor}{ExcludeCGI})
+	{
+		$conf{LogProcessor}{ExcludeCGI}=1; 
+	}	
 	# parameter muting the final report if nothing interesting to say
 	if (!exists $conf{LogProcessor}{QuietIfNoReport})
 	{
